import { __awaiter } from 'tslib';
import { Injectable, Inject, ɵɵdefineInjectable, ɵɵinject, Directive, ElementRef, ViewContainerRef, Input, Component, ComponentFactoryResolver, ViewChildren, NgModule } from '@angular/core';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @return {?}
 */
function _window() {
    return window;
}
class WindowRef {
    //Return the native Javascript window object
    /**
     * @return {?}
     */
    get nativeWindow() {
        return _window();
    }
}
WindowRef.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MagnoliaContextService {
    /**
     * Constructor
     * @param {?} winRef
     * @param {?} http
     * @param {?} environment
     */
    constructor(winRef, http, environment) {
        this.winRef = winRef;
        this.http = http;
        this.environment = environment;
        this.nativeWindow = winRef.nativeWindow;
        this.singlePageConfig = this.nativeWindow.singlePageConfig;
        this.fragmentURL = "/";
        if (this.singlePageConfig == null) {
            this.singlePageConfig = {};
        }
        else {
            this.contentNode = Promise.resolve(this.singlePageConfig.content);
        }
    }
    /**
     * Set the fragment URL.
     *
     * @param {?} fragment
     * @return {?}
     */
    setFragmentURL(fragment) {
        //If production mode, then gather content through a REST call
        if (!this.environment.mgnEditMode) {
            this.fragmentURL = fragment;
            this.contentNode = this.retrievePageContent();
        }
    }
    /**
     * Return whether the page is in edition mode
     *
     * @return {?} Whether the page is in edition mode
     */
    isEditionMode() {
        return this.environment.mgnEditMode;
    }
    /**
     * Returns the current Magnolia JCR node path.
     * @return {?}
     */
    getCurrentNode() {
        return __awaiter(this, void 0, void 0, /** @this {!MagnoliaContextService} */ function* () {
            /** @type {?} */
            let content = yield this.contentNode;
            return content["@path"];
        });
    }
    /**
     * Return the the defined area in the Magnolia context
     *
     * @param {?} areaName The name of the area
     * @return {?}
     */
    getAreaContent(areaName) {
        return __awaiter(this, void 0, void 0, /** @this {!MagnoliaContextService} */ function* () {
            /** @type {?} */
            let content = yield this.contentNode;
            return content[areaName];
        });
    }
    /**
     * Return the area's defined components
     *
     * @param {?} areaName The name of the area
     * @return {?}
     */
    getAreaComponents(areaName) {
        return __awaiter(this, void 0, void 0, /** @this {!MagnoliaContextService} */ function* () {
            /** @type {?} */
            let results = new Array();
            /** @type {?} */
            let content = yield this.contentNode;
            ;
            //Gets the area content
            /** @type {?} */
            let areaContent = content[areaName];
            if (areaContent != null) {
                /** @type {?} */
                let components = areaContent["@nodes"];
                components.map((/**
                 * @param {?} nodeName
                 * @return {?}
                 */
                nodeName => {
                    /** @type {?} */
                    let value = areaContent[nodeName];
                    if (typeof (value) === "object" && value["@nodeType"] === "mgnl:component") {
                        results.push(value);
                    }
                }));
            }
            return results;
        });
    }
    /**
     * Return the component mapping
     *
     * @param {?} templateId
     * @return {?}
     */
    getComponentMapping(templateId) {
        if (this.singlePageConfig.componentMappings !== null) {
            return this.singlePageConfig.componentMappings[templateId];
        }
        else {
            return null;
        }
    }
    /**
     * Return the HTML comment
     *
     * @param {?} nodePath The node path
     * @return {?}
     */
    getHtmlComment(nodePath) {
        if (this.singlePageConfig.htmlComments !== null) {
            return this.singlePageConfig.htmlComments[nodePath];
        }
        else {
            return null;
        }
    }
    /**
     * Return the page content JSON
     * @private
     * @return {?}
     */
    retrievePageContent() {
        return __awaiter(this, void 0, void 0, /** @this {!MagnoliaContextService} */ function* () {
            this.singlePageConfig.componentMappings = yield this.retrieveComponentMappings();
            return this.http.get(this.environment.restUrl + this.fragmentURL).pipe(map((/**
             * @param {?} response
             * @return {?}
             */
            response => {
                //console.log("retrievePageContent with " + this.fragmentURL + " value " + JSON.stringify(response,
                // null, 2));
                return response;
            }))).toPromise();
        });
    }
    /**
     * Return the fragment content JSON
     * @private
     * @return {?}
     */
    retrieveComponentMappings() {
        return __awaiter(this, void 0, void 0, /** @this {!MagnoliaContextService} */ function* () {
            return this.http.get(this.environment.configRestUrl + this.fragmentURL).pipe(map((/**
             * @param {?} response
             * @return {?}
             */
            response => {
                //console.log("retrieveComponentMappings with " + this.fragmentURL + " value " +
                // JSON.stringify(response, null, 2));
                return response;
            }))).toPromise();
        });
    }
}
MagnoliaContextService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
MagnoliaContextService.ctorParameters = () => [
    { type: WindowRef },
    { type: HttpClient },
    { type: undefined, decorators: [{ type: Inject, args: ['env',] }] }
];
/** @nocollapse */ MagnoliaContextService.ngInjectableDef = ɵɵdefineInjectable({ factory: function MagnoliaContextService_Factory() { return new MagnoliaContextService(ɵɵinject(WindowRef), ɵɵinject(HttpClient), ɵɵinject("env")); }, token: MagnoliaContextService, providedIn: "root" });
if (false) {
    /**
     * Native window object.
     * @type {?}
     */
    MagnoliaContextService.prototype.nativeWindow;
    /**
     * Single page config.
     * @type {?}
     */
    MagnoliaContextService.prototype.singlePageConfig;
    /**
     * Fragment URL.
     * @type {?}
     */
    MagnoliaContextService.prototype.fragmentURL;
    /**
     * Content node.
     * @type {?}
     * @private
     */
    MagnoliaContextService.prototype.contentNode;
    /**
     * @type {?}
     * @private
     */
    MagnoliaContextService.prototype.winRef;
    /**
     * @type {?}
     * @private
     */
    MagnoliaContextService.prototype.http;
    /**
     * @type {?}
     * @private
     */
    MagnoliaContextService.prototype.environment;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function MagnoliaComponent() { }
if (false) {
    /**
     * Component config.
     * @type {?}
     */
    MagnoliaComponent.prototype.component;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ComponentDirective {
    /**
     * Constructor.
     * @param {?} el
     * @param {?} mgnCtxService
     * @param {?} viewContainerRef
     */
    constructor(el, mgnCtxService, viewContainerRef) {
        this.el = el;
        this.mgnCtxService = mgnCtxService;
        this.viewContainerRef = viewContainerRef;
        this.nativeElement = el.nativeElement;
    }
    /**
     * On init
     * @return {?}
     */
    ngOnInit() {
        if (this.mgnCtxService.isEditionMode()) {
            //Get the HTML comment
            /** @type {?} */
            let htmlComment = this.mgnCtxService.getHtmlComment(this.component['@path']);
            //Get parent
            /** @type {?} */
            let parentDiv = this.nativeElement.parentNode;
            //Create comment before
            /** @type {?} */
            let commentBefore = document.createComment(htmlComment.start);
            parentDiv.appendChild(commentBefore);
            //Create comment after
            /** @type {?} */
            let commentAfter = document.createComment(htmlComment.end);
            parentDiv.appendChild(commentAfter);
            //Insert comment before
            parentDiv.insertBefore(commentBefore, this.nativeElement);
            //Insert comment after
            parentDiv.insertBefore(commentAfter, this.nativeElement.nextSibling);
        }
    }
}
ComponentDirective.decorators = [
    { type: Directive, args: [{
                selector: '[cmsComponent]'
            },] }
];
/** @nocollapse */
ComponentDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: MagnoliaContextService },
    { type: ViewContainerRef }
];
ComponentDirective.propDecorators = {
    component: [{ type: Input }]
};
if (false) {
    /**
     * Native element.
     * @type {?}
     */
    ComponentDirective.prototype.nativeElement;
    /**
     * Component config.
     * @type {?}
     */
    ComponentDirective.prototype.component;
    /**
     * @type {?}
     * @private
     */
    ComponentDirective.prototype.el;
    /**
     * @type {?}
     * @private
     */
    ComponentDirective.prototype.mgnCtxService;
    /** @type {?} */
    ComponentDirective.prototype.viewContainerRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AbstractArea {
    /**
     * Constructor
     * @param {?} elementRef
     * @param {?} mgnCtxService
     * @param {?} resolver
     */
    constructor(elementRef, mgnCtxService, resolver) {
        this.elementRef = elementRef;
        this.mgnCtxService = mgnCtxService;
        this.resolver = resolver;
    }
    /**
     * On init
     * @return {?}
     */
    ngOnInit() {
        this.mgnCtxService.getAreaComponents(this.cmsArea).then((/**
         * @param {?} data
         * @return {?}
         */
        data => {
            this.components = data;
        }));
    }
    /**
     * Loads component after init
     * @return {?}
     */
    ngAfterViewInit() {
        /** @type {?} */
        let current = this;
        //Attempt to load directly after init
        this.loadComponents(current);
        //Otherwise, observe the changes on the list of templates
        this.templates.changes.subscribe((/**
         * @return {?}
         */
        () => this.loadComponents(current)));
    }
    /**
     * Loads the components dynamically.
     *
     * @param {?} current The current area
     * @return {?}
     */
    loadComponents(current) {
        current.templates.forEach((/**
         * @param {?} item
         * @param {?} index
         * @param {?} array
         * @return {?}
         */
        function (item, index, array) {
            //Gets the component
            /** @type {?} */
            let component = item.component;
            // Gets the component class
            /** @type {?} */
            let componentClass = current.mgnCtxService.getComponentMapping(component["mgnl:template"]);
            if (typeof componentClass !== "undefined") {
                setTimeout((/**
                 * @return {?}
                 */
                () => {
                    current.loadComponent(componentClass, item, component);
                }));
            }
        }));
    }
    /**
     * Loads the components thanks to its cmsTemplateId
     * @param {?} componentClassName
     * @param {?} item
     * @param {?} component
     * @return {?}
     */
    loadComponent(componentClassName, item, component) {
        /** @type {?} */
        let factories = Array.from(this.resolver['_factories'].keys());
        /** @type {?} */
        let componentClass = factories.find((/**
         * @param {?} x
         * @return {?}
         */
        (x) => (x.key === componentClassName || x.name === componentClassName)));
        //Gets the component factory
        /** @type {?} */
        let componentFactory = this.resolver.resolveComponentFactory((/** @type {?} */ (componentClass)));
        //Get the view container
        item.viewContainerRef.clear();
        /** @type {?} */
        let componentRef = item.viewContainerRef.createComponent(componentFactory);
        ((/** @type {?} */ (componentRef.instance))).component = component;
    }
}
AbstractArea.decorators = [
    { type: Component, args: [{
                template: ""
            }] }
];
/** @nocollapse */
AbstractArea.ctorParameters = () => [
    { type: ElementRef },
    { type: MagnoliaContextService },
    { type: ComponentFactoryResolver }
];
AbstractArea.propDecorators = {
    cmsArea: [{ type: Input }],
    templates: [{ type: ViewChildren, args: [ComponentDirective,] }]
};
if (false) {
    /**
     * Area name.
     * @type {?}
     */
    AbstractArea.prototype.cmsArea;
    /**
     * List of area's components.
     * @type {?}
     */
    AbstractArea.prototype.components;
    /**
     * List of the component view containers.
     * @type {?}
     */
    AbstractArea.prototype.templates;
    /**
     * @type {?}
     * @private
     */
    AbstractArea.prototype.elementRef;
    /**
     * @type {?}
     * @private
     */
    AbstractArea.prototype.mgnCtxService;
    /**
     * @type {?}
     * @private
     */
    AbstractArea.prototype.resolver;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListArea extends AbstractArea {
}
ListArea.decorators = [
    { type: Component, args: [{
                selector: "[list-area]",
                template: "<ng-template ngFor let-component [ngForOf]=\"components\">\n    <ng-template [component]=\"component\" cmsComponent></ng-template>\n</ng-template>"
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AreaDirective {
    /**
     * Constructor.
     * @param {?} el
     * @param {?} mgnCtxService
     */
    constructor(el, mgnCtxService) {
        this.el = el;
        this.mgnCtxService = mgnCtxService;
    }
    /**
     * On init
     * @return {?}
     */
    ngOnInit() {
        if (this.mgnCtxService.isEditionMode()) {
            //Get the area node
            this.mgnCtxService.getAreaContent(this.cmsArea).then((/**
             * @param {?} areaNode
             * @return {?}
             */
            areaNode => {
                //Get the HTML comment
                /** @type {?} */
                let htmlComment = this.mgnCtxService.getHtmlComment(areaNode["@path"]);
                //Get parent
                /** @type {?} */
                let parentDiv = this.el.nativeElement.parentNode;
                //Create comment before
                /** @type {?} */
                let commentBefore = document.createComment(htmlComment.start);
                parentDiv.appendChild(commentBefore);
                //Create comment after
                /** @type {?} */
                let commentAfter = document.createComment(htmlComment.end);
                parentDiv.appendChild(commentAfter);
                //Insert comment before
                parentDiv.insertBefore(commentBefore, this.el.nativeElement);
                //Insert comment after
                parentDiv.insertBefore(commentAfter, this.el.nativeElement.nextSibling);
            }));
        }
    }
}
AreaDirective.decorators = [
    { type: Directive, args: [{
                selector: '[cmsArea]'
            },] }
];
/** @nocollapse */
AreaDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: MagnoliaContextService }
];
AreaDirective.propDecorators = {
    cmsArea: [{ type: Input }]
};
if (false) {
    /**
     * Area name.
     * @type {?}
     */
    AreaDirective.prototype.cmsArea;
    /**
     * @type {?}
     * @private
     */
    AreaDirective.prototype.el;
    /**
     * @type {?}
     * @private
     */
    AreaDirective.prototype.mgnCtxService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularMagnoliaModule {
    /**
     * @param {?} environment
     * @return {?}
     */
    static forRoot(environment) {
        return {
            ngModule: AngularMagnoliaModule,
            providers: [
                WindowRef,
                {
                    provide: 'env',
                    useValue: environment
                }
            ]
        };
    }
}
AngularMagnoliaModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, HttpClientModule],
                declarations: [
                    AreaDirective,
                    ComponentDirective,
                    AbstractArea,
                    ListArea
                ],
                providers: [
                    WindowRef
                ],
                exports: [
                    AreaDirective,
                    ListArea,
                    ComponentDirective
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AbstractArea, AngularMagnoliaModule, AreaDirective, ComponentDirective, ListArea, MagnoliaContextService, WindowRef as ɵa };
//# sourceMappingURL=angular-magnolia.js.map
