import { HttpClient } from '@angular/common/http';
import { WindowRef } from './windowref.service';
export declare class MagnoliaContextService {
    private winRef;
    private http;
    private environment;
    /** Native window object. */
    nativeWindow: any;
    /** Single page config.*/
    singlePageConfig: any;
    /** Fragment URL.*/
    fragmentURL: string;
    /** Content node. */
    private contentNode;
    /**
     * Constructor
     */
    constructor(winRef: WindowRef, http: HttpClient, environment: any);
    /**
     * Set the fragment URL.
     *
     * @param fragement URL
     */
    setFragmentURL(fragment: string): void;
    /**
     * Return whether the page is in edition mode
     *
     * @return Whether the page is in edition mode
     */
    isEditionMode(): boolean;
    /**
     * Returns the current Magnolia JCR node path.
     */
    getCurrentNode(): Promise<any>;
    /**
     * Return the the defined area in the Magnolia context
     *
     * @param areaName The name of the area
     */
    getAreaContent(areaName: string): Promise<any>;
    /**
     * Return the area's defined components
     *
     * @param areaName The name of the area
     */
    getAreaComponents(areaName: string): Promise<any[]>;
    /**
     * Return the component mapping
     *
     * @param nodePath The component mapping
     */
    getComponentMapping(templateId: string): any;
    /**
     * Return the HTML comment
     *
     * @param nodePath The node path
     */
    getHtmlComment(nodePath: string): any;
    /**
     * Return the page content JSON
     */
    private retrievePageContent;
    /**
     * Return the fragment content JSON
     */
    private retrieveComponentMappings;
}
